==> fred-hermes-2.0.0.tar.gz: fred-hermes-2.0.0/hermes/tests/test_message.py <==
"""Unittests for generic message structures."""
from typing import Any, Dict
from unittest import TestCase
from unittest.mock import Mock, call, sentinel
from uuid import UUID

from typist import SecretaryClient
from typist.client import TemplateNotFound

from hermes.message import Message, MessageEnvelope, Reference, ReferenceType


class TestMessage(Message):
    """Test message."""

    rendered_fields = ('subject',)

    @property
    def subject(self) -> str:
        return self._secretary_render('subject', template='template.txt')

    @classmethod
    def from_dict(cls, data: Dict[str, Any], *, secretary_client: SecretaryClient = None) -> 'TestMessage':
        message = cls()
        message.secretary_client = secretary_client
        return message


class TestMessageEnvelope(MessageEnvelope):
    """Test message envelope."""

    message_cls = TestMessage


class MessageTest(TestCase):
    """Unittests for message."""

    def test_init_empty(self):
        message = TestMessage()
        self.assertIsNone(message.type)
        self.assertIsNone(message.secretary_client)

    def test_init_values(self):
        message = TestMessage(type=sentinel.type, secretary_client=sentinel.secretary_client)
        self.assertEqual(message.type, sentinel.type)
        self.assertEqual(message.secretary_client, sentinel.secretary_client)

    def test_secretary_render(self):
        """Test _secretary_render without secretary client."""
        message = TestMessage()
        with self.assertRaises(RuntimeError):
            message._secretary_render(
                sentinel.name,
                template=sentinel.template,
                render_method_name=sentinel.render_method_name,
            )

    def test_to_dict(self):
        message = TestMessage(type=sentinel.type)
        self.assertEqual(message.to_dict(), {
            'type': sentinel.type,
        })


class MessageWithSecretaryClientTest(TestCase):
    """Unittests for message structure with a secretary client."""

    def setUp(self):
        self.secretary_client = Mock(autospec=SecretaryClient)
        self.message = TestMessage(
            type=sentinel.type,
            secretary_client=self.secretary_client,
        )

    def test_secretary_render(self):
        """Test _secretary_render with secretary client."""
        self.secretary_client.render.return_value = sentinel.subject
        self.assertEqual(self.message.subject, sentinel.subject)
        self.assertEqual(self.message.subject, sentinel.subject)  # call subject again to test the caching
        self.assertEqual(
            self.secretary_client.mock_calls,
            [call.render('template.txt', context={})]
        )

    def test_to_dict(self):
        self.secretary_client.render.return_value = sentinel.subject
        self.assertEqual(self.message.to_dict(), {
            'type': sentinel.type,
            'subject': sentinel.subject,
        })

    def test_get_secretary_client_data_with_errors(self):
        for error_cls in [RuntimeError, TemplateNotFound, ValueError]:
            with self.subTest(error_cls=error_cls):
                self.secretary_client.render.side_effect = error_cls('Dangerous error')
                with self.assertLogs('hermes.message', level='WARNING') as log:
                    data = self.message._get_secretary_client_data()
                self.assertEqual(data, {
                    'subject': None,
                    'render_errors': [
                        'SecretaryClient raised an error in the "subject" property: Dangerous error',
                    ],
                })
                self.assertEqual(log.output, [
                    'WARNING:hermes.message:SecretaryClient raised an error in the "subject" property: Dangerous error',
                ])

    def test_secretary_client_fields_error_raised(self):
        self.secretary_client.render.side_effect = TemplateNotFound()
        # call twice to test nothing has been cached
        with self.assertRaises(TemplateNotFound):
            self.message.subject
        with self.assertRaises(TemplateNotFound):
            self.message.subject
        self.assertEqual(
            self.secretary_client.mock_calls,
            [call.render('template.txt', context={}), call.render('template.txt', context={})]
        )


class MessageEnvelopeTest(TestCase):
    """Unittests for MessageEnvelope structure."""

    def test_init_empty(self):
        message = TestMessage()
        message_envelope = TestMessageEnvelope(message)
        self.assertIsNone(message_envelope.uid)
        self.assertIsNone(message_envelope.create_datetime)
        self.assertIsNone(message_envelope.send_datetime)
        self.assertIsNone(message_envelope.delivery_datetime)
        self.assertEqual(message_envelope.attempts, 0)
        self.assertIsNone(message_envelope.status)
        self.assertEqual(message_envelope.message, message)
        self.assertTrue(message_envelope.archive)
        self.assertEqual(message_envelope.references, [])

    def test_init_values(self):
        message_envelope = TestMessageEnvelope(
            sentinel.message, uid=sentinel.uid, create_datetime=sentinel.create_datetime,
            send_datetime=sentinel.send_datetime, delivery_datetime=sentinel.delivery_datetime,
            attempts=sentinel.attempts, status=sentinel.status, archive=False, references=sentinel.references,
        )
        self.assertEqual(message_envelope.uid, sentinel.uid)
        self.assertEqual(message_envelope.create_datetime, sentinel.create_datetime)
        self.assertEqual(message_envelope.send_datetime, sentinel.send_datetime)
        self.assertEqual(message_envelope.delivery_datetime, sentinel.delivery_datetime)
        self.assertEqual(message_envelope.attempts, sentinel.attempts)
        self.assertEqual(message_envelope.status, sentinel.status)
        self.assertEqual(message_envelope.message, sentinel.message)
        self.assertFalse(message_envelope.archive)
        self.assertEqual(message_envelope.references, sentinel.references)

    def test_from_dict_data(self):
        data = {
            'uid': sentinel.uid,
            'create_datetime': sentinel.create_datetime,
            'send_datetime': sentinel.send_datetime,
            'delivery_datetime': sentinel.delivery_datetime,
            'attempts': sentinel.attempts,
            'status': sentinel.status,
            'message_data': {},
            'archive': False,
            'references': sentinel.references,
        }
        message_envelope = TestMessageEnvelope.from_dict(data, secretary_client=sentinel.secretary_client)
        self.assertIsInstance(message_envelope.message, TestMessage)
        self.assertEqual(message_envelope.uid, sentinel.uid)
        self.assertEqual(message_envelope.create_datetime, sentinel.create_datetime)
        self.assertEqual(message_envelope.send_datetime, sentinel.send_datetime)
        self.assertEqual(message_envelope.delivery_datetime, sentinel.delivery_datetime)
        self.assertEqual(message_envelope.attempts, sentinel.attempts)
        self.assertEqual(message_envelope.status, sentinel.status)
        self.assertFalse(message_envelope.archive)
        self.assertEqual(message_envelope.message.secretary_client, sentinel.secretary_client)
        self.assertEqual(message_envelope.references, sentinel.references)
        self.assertIn('message_data', data)

    def test_to_dict(self):
        data = {
            'uid': sentinel.uid,
            'create_datetime': sentinel.create_datetime,
            'status': sentinel.status,
            'message_data': {},
            'archive': False,
            'references': [Reference(value=UUID(int=1), type=ReferenceType.CONTACT)],
        }
        message_envelope = TestMessageEnvelope.from_dict(data)
        self.assertEqual(message_envelope.to_dict(), {
            'uid': sentinel.uid,
            'create_datetime': sentinel.create_datetime,
            'send_datetime': None,
            'delivery_datetime': None,
            'message': message_envelope.message.to_dict(),
            'attempts': 0,
            'status': sentinel.status,
            'archive': False,
            'references': [{'value': UUID(int=1), 'type': 'contact'}],
        })

==> fred-hermes-4.1.0.tar.gz: fred-hermes-4.1.0/hermes/tests/test_message.py <==
"""Unittests for generic message structures."""
from unittest import TestCase
from unittest.mock import call, patch, sentinel

from typist import SecretaryClient
from typist.client import TemplateNotFound

from hermes.message import Message, MessageEnvelope


class TestMessage(Message):
    """Test message."""

    data: str
    render_fields = ('subject',)

    @property
    def subject(self) -> str:
        return self._secretary_render('subject', template='template.txt')


class TestMessageEnvelope(MessageEnvelope):
    """Test message envelope."""

    message_cls = TestMessage


class MessageTest(TestCase):
    """Unittests for message."""

    def test_dict(self):
        message = TestMessage(data='gazpacho', type='T')
        self.assertEqual(message.dict(), {'data': 'gazpacho', 'type': 'T'})

    def test_secretary_render(self):
        """Test _secretary_render without secretary client."""
        message = TestMessage(data='gazpacho')
        with self.assertRaises(RuntimeError):
            message._secretary_render(
                sentinel.name,
                template=sentinel.template,
                render_method_name=sentinel.render_method_name,
            )


class MessageWithSecretaryClientTest(TestCase):
    """Unittests for message structure with a secretary client."""

    def setUp(self):
        self.secretary_client = SecretaryClient('localhost')
        patcher = patch.object(self.secretary_client, 'render')
        self.addCleanup(patcher.stop)
        patcher.start()
        self.message = TestMessage(data='gazpacho', type='T')
        self.message._secretary_client = self.secretary_client

    def test_secretary_render(self):
        """Test _secretary_render with secretary client."""
        self.secretary_client.render.return_value = sentinel.subject
        self.assertEqual(self.message.subject, sentinel.subject)
        self.assertEqual(self.message.subject, sentinel.subject)  # call subject again to test the caching
        self.assertEqual(
            self.secretary_client.render.mock_calls,
            [call('template.txt', context={})]
        )

    def test_dict(self):
        self.secretary_client.render.return_value = sentinel.subject
        self.assertEqual(self.message.dict(), {
            'data': 'gazpacho',
            'type': 'T',
            'subject': sentinel.subject,
        })

    def test_get_secretary_client_data_with_errors(self):
        for error_cls in [RuntimeError, TemplateNotFound, ValueError]:
            with self.subTest(error_cls=error_cls):
                self.secretary_client.render.side_effect = error_cls('Dangerous error')
                with self.assertLogs('hermes.message', level='WARNING') as log:
                    data = self.message._get_secretary_client_data()
                self.assertEqual(data, {
                    'subject': None,
                    'render_errors': [
                        'SecretaryClient raised an error in the "subject" property: Dangerous error',
                    ],
                })
                self.assertEqual(log.output, [  # type: ignore[union-attr]
                    'WARNING:hermes.message:SecretaryClient raised an error in the "subject" property: Dangerous error',
                ])

    def test_secretary_client_fields_error_raised(self):
        self.secretary_client.render.side_effect = TemplateNotFound
        # call twice to test nothing has been cached
        with self.assertRaises(TemplateNotFound):
            self.message.subject
        with self.assertRaises(TemplateNotFound):
            self.message.subject
        self.assertEqual(
            self.secretary_client.render.mock_calls,
            [call('template.txt', context={}), call('template.txt', context={})]
        )
